# -- coding: utf-8 --
"""
This module contains tools for rasterizing vector data.
"""
import numpy
import rasterio.features
import rasterio.transform
import rasterio.warp

from geocube.logger import get_logger


def rasterize_image(geojson_shapes, data_values, geobox, fill=-9999.0):
    """
    Rasterize a list of shapes+values for a given GeoBox.

    Parameters
    -----------
    geojson_shapes: list
        List of geojson shapes to rasterize.
    data_values: list
        Data values associated with the list of geojson shapes
    geobox: :obj:`datacube.utils.geometry.GeoBox`
        Transform of the resulting image.
    fill: float, optional
        The value to fill in the grid with for nodata. Default is -9999.0.

    Returns
    -------
    :obj:`numpy.ndarray` or None
        The vector data in the rasterized format.

    """
    logger = get_logger()

    try:
        image = rasterio.features.rasterize(
            zip(geojson_shapes, data_values),
            out_shape=(geobox.height, geobox.width),
            transform=geobox.affine,
            fill=fill,
            dtype=numpy.float64,
        )
        return image
    except TypeError as ter:
        if "cannot perform reduce with flexible type" in str(ter):
            logger.warning("{warning}".format(warning=ter))
            return None
        raise



# -- coding: utf-8 --
"""
This module contains helper methods for rasterio.crs.CRS.
"""


def crs_to_wkt(rasterio_crs):
    """
    This is to deal with change in rasterio.crs.CRS.

    Parameters
    ----------
    rasterio_crs: :obj:`rasterio.crs.CRS`
        Rasterio object.

    Returns
    -------
    str: WKT string.

    """
    try:
        # rasterio>=1.0.14
        return rasterio_crs.to_wkt()
    except AttributeError:
        return rasterio_crs.wkt



# -- coding: utf-8 --
"""
GeoCube client core functionality
"""
from geocube.geo_utils.geobox import GeoBoxMaker
from geocube.vector_to_cube import VectorToCube


def make_geocube(
    vector_data,  # pylint: disable=too-many-arguments
    measurements=None,
    datetime_measurements=None,
    output_crs=None,
    resolution=None,
    align=None,
    geom=None,
    like=None,
    fill=None,
    group_by=None,
    interpolate_na_method=None,
    categorical_enums=None,
):  # NOSONAR
    """
    Rasterize vector data into an ``xarray`` object.  Each attribute will be a data
    variable in the :class:`xarray.Dataset`.

    Parameters
    ----------
    vector_data: str or :obj:`geopandas.GeoDataFrame`
        A file path to an OGR supported source or GeoDataFrame containing
        the vector data.
    measurements: list(str), optional
        Attributes name or list of names to be included. If a list is specified,
        the attributes will be returned in the order requested.
        By default all available attributes are included.
    datetime_measurements: list(str), optional
        Attributes that are temporal in nature and should be converted to the datetime
        format. These are only included if listed in 'measurements'.
    output_crs: str, optional
        The CRS of the returned data.  If no CRS is supplied, the CRS of the
        stored data is used.
    resolution: (float,float), optional
        A tuple of the spatial resolution of the returned data.
        This includes the direction (as indicated by a positive or negative number).
        Typically when using most CRSs, the first number would be negative.
    align: (float,float), optional
        Load data such that point 'align' lies on the pixel boundary.
        Units are in the co-ordinate space of the output CRS.
        Default is (0,0)
    geom: str, optional
        A GeoJSON string for the bounding box of the data used to construct the
        grid.
    like: :obj:`xarray.Dataset`, optional
        Uses the output of a previous ``load()`` to form the basis of a request
        for another product.
        E.g.::

            gcds = make_geocube(vector_data='my_vector.geopackage', like=other_gcds)

    fill: float, optional
        The value to fill in the grid with for nodata. Default is -9999.0.
    group_by: str, optional
        When specified, perform basic combining/reducing of the data on this column.
    interpolate_na_method:  {‘linear’, ‘nearest’, ‘cubic’}, optional
        This is the method for interpolation to use to fill in the nodata with
        :meth:`scipy.interpolate.griddata`.
    categorical_enums: dict, optional
        A dictionary of all categories for the table columns containing categorical
        data.
        E.g. {'column_name': ['a', 'b'], 'other_column': ['c', 'd']}

    Returns
    --------
    :obj:`xarray.Dataset`:
        Requested data in a :obj:`xarray.Dataset`.

    """
    geobox_maker = GeoBoxMaker(output_crs, resolution, align, geom, like)

    return VectorToCube(
        vector_data=vector_data,
        geobox_maker=geobox_maker,
        fill=fill,
        categorical_enums=categorical_enums,
    ).make_geocube(
        measurements=measurements,
        datetime_measurements=datetime_measurements,
        group_by=group_by,
        interpolate_na_method=interpolate_na_method,
    )



from mock import patch

from geocube.logger import log_to_console, log_to_file, get_logger


def test_log_to_console(capsys):
    log_to_console(False)  # clear any other loggers
    log_to_console(level="INFO")
    logm = get_logger()
    logm.info("here")
    captured = capsys.readouterr()
    assert "INFO-geocube: here" in captured.err


def test_log_to_console__warning(capsys):
    log_to_console(False)  # clear any other loggers
    log_to_console()
    logm = get_logger()
    logm.info("here")
    logm.warning("there")
    captured = capsys.readouterr()
    assert "INFO-geocube: here" not in captured.err
    assert "WARNING-geocube: there" in captured.err


@patch("geocube.logger.appdirs.user_log_dir")
def test_log_to_file(mock_user_log_dir, tmpdir):
    mock_user_log_dir.return_value = str(tmpdir)
    log_to_file(False)  # clear any other loggers
    log_to_file(level="INFO")
    logm = get_logger()
    logm.info("here")
    with open(tmpdir.join("geocube.log")) as logf:
        captured = logf.read()
    assert "INFO-geocube: here" in captured


@patch("geocube.logger.appdirs.user_log_dir")
def test_log_to_file__warning(mock_user_log_dir, tmpdir):
    mock_user_log_dir.return_value = str(tmpdir)
    log_to_file(False)  # clear any other loggers
    log_to_file()
    logm = get_logger()
    logm.info("here")
    logm.warning("there")
    with open(tmpdir.join("geocube.log")) as logf:
        captured = logf.read()
    assert "INFO-geocube: here" not in captured
    assert "WARNING-geocube: there" in captured



import os

import geopandas as gpd
import pytest
import xarray
from dateutil.parser import parse
from numpy.testing import assert_array_equal

from geocube.xarray_extensions import vectorxarray
from test.conftest import TEST_INPUT_DATA_DIR


def assert_test_dataframes_equal(gdf, gdf2):
    assert gdf.crs == gdf2.crs
    assert sorted(gdf.columns) == sorted(gdf2.columns)
    for column in gdf.columns:
        assert_array_equal(gdf[column], gdf2[column])


def test_from_geodataframe():
    gdf = gpd.read_file(os.path.join(TEST_INPUT_DATA_DIR, "soil_data_flat.geojson"))
    vxd = vectorxarray.from_geodataframe(gdf)
    assert all(gdf.geometry == vxd.geometry.values)
    assert sorted(gdf.columns.tolist() + ["crs"]) == sorted(vxd.variables)
    assert gdf.crs == vxd.crs.attrs
    assert "geometry" in vxd.coords
    assert "crs" in vxd.coords


def test_to_geodataframe():
    gdf = gpd.read_file(os.path.join(TEST_INPUT_DATA_DIR, "soil_data_flat.geojson"))
    gdf2 = vectorxarray.from_geodataframe(gdf).vector.to_geodataframe()
    assert_test_dataframes_equal(gdf, gdf2)


def test_to_netcdf(tmpdir):
    gdf = gpd.read_file(os.path.join(TEST_INPUT_DATA_DIR, "soil_data_flat.geojson"))
    vxd = vectorxarray.from_geodataframe(gdf)
    output_file = tmpdir.join("test_vector.nc")
    vxd.vector.to_netcdf(output_file)
    vxd2 = vectorxarray.open_dataset(str(output_file))
    assert_test_dataframes_equal(gdf, vxd2.vector.to_geodataframe())


def test_multidimensional_error():
    gdf = gpd.read_file(os.path.join(TEST_INPUT_DATA_DIR, "soil_data_flat.geojson"))
    vxd = vectorxarray.from_geodataframe(gdf)
    vxd2 = vxd.copy()
    vxd.coords["time"] = parse("20170516T000000")
    vxd2.coords["time"] = parse("20170517T000000")
    merged_vxd = xarray.concat([vxd, vxd2], dim="time")
    with pytest.raises(ValueError):
        merged_vxd.vector.plot(column="sandtotal_r")



# -- coding: utf-8 --
"""
The CLI interface to the load module for GeoCube
"""

import click
import xarray

from geocube.api import core


@click.command(name="make-geocube")
@click.option(
    "-i",
    "--interpolate-na-method",
    help="This is the method for interpolation to use to fill in the nodata.",
    type=click.Choice(["linear", "nearest", "cubic"]),
    required=False,
)
@click.option(
    "-gb",
    "--group-by",
    help="When specified, perform basic combining/reducing of the data on this column.",
    required=False,
)
@click.option(
    "-f",
    "--fill",
    type=float,
    help="The value to fill in the grid with for nodata. Default is -9999.0.",
    required=False,
)
@click.option(
    "-l",
    "--like",
    type=click.Path(exists=True),
    help="Uses the netCDF file output of a previous ``load()`` "
    "to form the basis of a request for another product.",
    required=False,
)
@click.option(
    "-g",
    "--geom",
    help="GeoJSON string for the bounding box of the data used to construct the grid.",
    required=False,
)
@click.option(
    "-a",
    "--align",
    nargs=2,
    type=float,
    help=(
        "Load data such that point 'align' lies on the pixel boundary."
        " Default is (0, 0)."
    ),
    required=False,
)
@click.option(
    "-r",
    "--resolution",
    nargs=2,
    type=float,
    help="A tuple of the spatial resolution of the returned data. Ex. -r -10 10.",
    required=False,
)
@click.option(
    "-c",
    "--output-crs",
    help=(
        "The CRS of the returned data. (e.x. +init=epsg:4326). "
        "If no CRS is supplied, the CRS of the input data is used."
    ),
    required=False,
)
@click.option(
    "-m",
    "--measurements",
    help="Measurement subset to load in.",
    multiple=True,
    required=False,
)
@click.argument("vector_data", type=click.Path(exists=True), required=True)
@click.argument("output_file", required=True)
def make_geocube(
    output_file,
    vector_data,
    measurements,
    output_crs,
    resolution,  # pylint: disable=too-many-arguments
    align,
    geom,
    like,
    fill,
    group_by,
    interpolate_na_method,
):  # NOSONAR
    """
    Utility to load vector data into the xarray raster format.
    """
    if not resolution and isinstance(resolution, tuple):
        resolution = None
    if not align and isinstance(align, tuple):
        align = None
    if not measurements and isinstance(measurements, tuple):
        measurements = None

    if like is not None:
        like = xarray.open_dataset(like, autoclose=True)
    try:
        gcds = core.make_geocube(
            vector_data=vector_data,
            measurements=measurements,
            output_crs=output_crs,
            resolution=resolution,
            align=align,
            geom=geom,
            like=like,
            fill=fill,
            group_by=group_by,
            interpolate_na_method=interpolate_na_method,
        )
    finally:
        if like is not None:
            like.close()

    gcds.to_netcdf(output_file)



# -- coding: utf-8 --
"""
geocube core conversion functionality
"""
import numpy
import pandas
import xarray
from shapely.geometry import mapping

from geocube.geo_utils.geobox import load_vector_data
from geocube.logger import get_logger
from geocube.rasterize import rasterize_image
from rioxarray.rioxarray import (
    DEFAULT_GRID_MAP,
    add_spatial_ref,
    add_xy_grid_meta,
    affine_to_coords,
)


def _format_series_data(data_series):
    """
    The purpose of this function is to convert the series data into a rasterizeable
    format if possible.

    Parameters
    ----------
    data_series: :obj:`geopandas.GeoSeries`
        The series to be converted.

    Returns
    -------
    :obj:`geopandas.GeoSeries`: The series that was converted if possible.

    """
    if "datetime" in str(data_series.dtype):
        data_series = pandas.to_numeric(data_series).astype(numpy.float64)
        get_logger().warning(
            "The series '{}' was converted from a date to a number to "
            "rasterize the data. To load the data back in as a date, "
            "use 'pandas.to_datetime()'.".format(data_series.name)
        )
    elif str(data_series.dtype) == "category":
        data_series = data_series.cat.codes
    return data_series


class VectorToCube(object):
    """
    Tool that facilitates converting vector data to raster data into
    an :obj:`xarray.DataFrame`.
    """

    def __init__(self, vector_data, geobox_maker, fill=None, categorical_enums=None):
        """
        Initialize the GeoCube class.

        Parameters
        ----------
        vector_data: str or :obj:`geopandas.GeoDataFrame`
            A file path to an OGR supported source or GeoDataFrame containing
            the vector data.
        geobox_maker: :obj:`geocube.geo_utils.geobox.GeoBoxMaker`
            The geobox for the grid to be generated from the vector data.
        fill: float, optional
            The value to fill in the grid with for nodata. Default is -9999.0.
        categorical_enums: dict, optional
            A dictionary of all categories for the table columns containing
            categorical data.
            E.g. {'column_name': ['a', 'b'], 'other_column': ['c', 'd']}

        """
        self.vector_data = load_vector_data(vector_data)
        self.geobox = geobox_maker.from_vector(self.vector_data)
        self.fill = fill if fill is not None else -9999.0
        if categorical_enums is not None:
            for column_name, categories in categorical_enums.items():
                category_type = pandas.api.types.CategoricalDtype(
                    categories=sorted(set(categories)) + ["nodata"]
                )
                self.vector_data[column_name] = self.vector_data[column_name].astype(
                    category_type
                )

    def make_geocube(
        self,  # pylint: disable=too-many-arguments
        measurements=None,
        datetime_measurements=None,
        group_by=None,
        interpolate_na_method=None,
    ):  # NOSONAR
        """
        Rasterize vector data into an ``xarray`` object.  Each measurement will be a
        data variable in the :class:`xarray.Dataset`.

        See the `xarray documentation <http://xarray.pydata.org>`_
        for usage of the :class:`xarray.Dataset` and :class:`xarray.DataArray` objects.

        Parameters
        ----------
        measurements: list(str), optional
            Attributes name or list of names to be included. If a list is specified,
            the measurements will be returned in the order requested.
            By default all available measurements are included.
        datetime_measurements: list(str), optional
            Attributes that are temporal in nature and should be converted to the
            datetime format. These are only included if listed in 'measurements'.
        group_by: str, optional
            When specified, perform basic combining/reducing of the data on this column.
        interpolate_na_method:  {‘linear’, ‘nearest’, ‘cubic’}, optional
            This is the method for interpolation to use to fill in the nodata with
            :meth:`scipy.interpolate.griddata`.

        Returns
        --------
        :class:`xarray.Dataset`:
            Requested data in a :class:`xarray.Dataset`.

        """
        if measurements is None:
            measurements = self.vector_data.columns.tolist()
            measurements.remove("geometry")

        self.datetime_measurements = ()
        if datetime_measurements is not None:
            self.datetime_measurements = tuple(
                set(datetime_measurements) & set(measurements)
            )
        # reproject vector data to the projection of the output raster
        vector_data = self.vector_data.to_crs(
            self.geobox.crs._crs.ExportToProj4()
        )  # pylint: disable=protected-access

        # convert to datetime
        for datetime_measurement in self.datetime_measurements:
            vector_data[datetime_measurement] = pandas.to_datetime(
                vector_data[datetime_measurement]
            )

        # get categorical enumerations if they exist
        self._categorical_enums = {}
        for categorical_column in vector_data.select_dtypes(["category"]).columns:
            self._categorical_enums[categorical_column] = vector_data[
                categorical_column
            ].cat.categories

        # map the shape data to the grid
        if group_by:
            vector_data = vector_data.groupby(group_by)
            try:
                measurements.remove(group_by)
            except ValueError:
                pass

        return self._get_dataset(
            vector_data, measurements, group_by, interpolate_na_method
        )

    @staticmethod
    def _get_attrs(measurement_name, fill_value):
        """
        Get attributes for data array.

        Parameters
        ----------
        measurement_name: str
            The measurement name.
        fill_value: int or fload
            The fill value.

        Returns
        -------
        dict: Dict with attributes for data array.
        """
        return dict(
            name=measurement_name,
            long_name=measurement_name,
            grid_mapping=DEFAULT_GRID_MAP,
            _FillValue=fill_value,
        )

    def _update_time_attrs(self, attrs, image_data):
        """
        Update attributes and nodata values for time grid.

        Parameters
        ----------
        attrs: dict
            Dict with attributes for data array.
        image_data: :obj:`numpy.ndarray`
            Array with image data.

        Returns
        -------
        dict: Dict with attributes for data array.
        """
        attrs["units"] = "seconds from 1970-01-01T00:00:00"
        attrs["_FillValue"] = 0
        image_data[image_data == self.fill] = 0.0

    def _get_dataset(self, vector_data, measurements, group_by, interpolate_na_method):
        """
        Parameters
        ----------
        vector_data: str or :obj:`geopandas.GeoDataFrame`
            A file path to an OGR supported source or GeoDataFrame containing
            the vector data.
        measurements: list(str), optional
            Attributes name or list of names to be included. If a list is specified,
            the measurements will be returned in the order requested.
            By default all available measurements are included.
        group_by: str, optional
            When specified, perform basic combining/reducing of the data on this column.

        Returns
        --------
        :obj:`xarray.Dataset`:
            Requested data in a :obj:`xarray.Dataset`.

        """
        data_vars = {}
        for measurement in measurements:
            if group_by:
                grid_array = self._get_grouped_grid(
                    vector_data[[measurement, "geometry"]], measurement, group_by
                )
            else:
                grid_array = self._get_grid(
                    vector_data[[measurement, "geometry"]], measurement
                )
            if grid_array is not None:
                data_vars[measurement] = grid_array

        coords = affine_to_coords(
            self.geobox.affine, self.geobox.width, self.geobox.height
        )

        if group_by:
            coords[group_by] = list(vector_data.groups.keys())

        out_xds = xarray.Dataset(data_vars=data_vars, coords=coords)

        for categorical_measurement, categoral_enums in self._categorical_enums.items():
            enum_var_name = "{}_categories".format(categorical_measurement)
            cat_attrs = dict(out_xds[categorical_measurement].attrs)
            cat_attrs["categorical_mapping"] = enum_var_name
            out_xds[categorical_measurement].attrs = cat_attrs
            out_xds[enum_var_name] = categoral_enums

        add_xy_grid_meta(out_xds.coords)
        add_spatial_ref(out_xds, self.geobox.crs.crs_str, DEFAULT_GRID_MAP)

        if interpolate_na_method is not None:
            return out_xds.rio.interpolate_na(method=interpolate_na_method)

        return out_xds

    def _get_grouped_grid(self, grouped_dataframe, measurement_name, group_by):
        """Retrieve the variable data to append to the ssurgo :obj:`xarray.Dataset`.
        This method is designed specifically to work on a dataframe that has
        been grouped.

        Parameters
        ----------
        grouped_dataframe: pandas GroupBy object
            A pandas dataframe in as a GroupBy object.
        measurement_name: str
            Attributes name or list of names to be included. If a list is specified,
            the measurements will be returned in the order requested.
            By default all available measurements are included.
        group_by: str
            When specified, perform basic combining/reducing of the data on this column.

        Returns
        -------
        tuple: Options needed to create an :obj:`xarray.DataArray`.

        """
        logger = get_logger()

        image_data = []
        df_group = None
        fill_value = self.fill
        for _, df_group in grouped_dataframe:
            fill_value = (
                self.fill if str(df_group[measurement_name].dtype) != "category" else -1
            )
            image = rasterize_image(
                df_group.geometry.apply(mapping).values,
                _format_series_data(df_group[measurement_name]).values,
                self.geobox,
                fill_value,
            )
            if image is None:
                logger.warning(
                    "Skipping attribute {col} due to missing data...".format(
                        col=measurement_name
                    )
                )
                return None

            image_data.append(image)

        attrs = self._get_attrs(measurement_name, fill_value)
        image_data = numpy.array(image_data)
        # it was converted to numeric date value
        if df_group is not None and "datetime" in str(df_group[measurement_name].dtype):
            self._update_time_attrs(attrs, image_data)

        return (group_by, "y", "x"), image_data, attrs

    def _get_grid(self, dataframe, measurement_name):
        """Retrieve the variable data to append to the ssurgo :obj:`xarray.Dataset`
        from a regular :obj:`geopandas.GeoDataFrame`.

        Parameters
        ----------
        dataframe: :obj:`geopandas.GeoDataFrame`
            A geopandas GeoDataFrame object to rasterize.
        measurement_name: str
            Attributes name or list of names to be included. If a list is specified,
            the measurements will be returned in the order requested.
            By default all available measurements are included.

        Returns
        -------
        tuple: Options needed to create an :obj:`xarray.DataArray`.

        """
        logger = get_logger()
        fill_value = (
            self.fill if str(dataframe[measurement_name].dtype) != "category" else -1
        )
        image_data = rasterize_image(
            dataframe.geometry.apply(mapping).values,
            _format_series_data(dataframe[measurement_name]).values,
            self.geobox,
            fill_value,
        )
        if image_data is None:
            logger.warning(
                "Skipping attribute {col} due to missing data...".format(
                    col=measurement_name
                )
            )
            return None

        attrs = self._get_attrs(measurement_name, fill_value)

        # it was converted to numeric date value
        if "datetime" in str(dataframe[measurement_name].dtype):
            self._update_time_attrs(attrs, image_data)

        return ("y", "x"), numpy.array(image_data), attrs



# -- coding: utf-8 --
"""
This module is an extension for xarray to provide support for vector datasets.
"""
import geopandas as gpd
import xarray
from shapely.wkb import dumps, loads


def from_geodataframe(in_geodataframe):
    """
    Create an xarray object from a geodataframe.
    """
    geodf = in_geodataframe.copy().set_index("geometry")
    geox = xarray.Dataset.from_dataframe(geodf)
    geox.coords["crs"] = 0
    geox.coords["crs"].attrs = geodf.crs
    return geox


def open_dataset(*args, **kwargs):
    """
    Open the vxarray exported netCDF file as a Dataset.

    Returns
    -------
    :obj:`xarray.Dataset`

    """
    xds = xarray.open_dataset(*args, **kwargs)
    xds.coords["geometry"] = list(map(loads, xds.coords["geometry"].values))
    return xds


class BaseVectorX:
    """This is the base for the vector GIS extension."""

    def __init__(self, xarray_obj):
        self._obj = xarray_obj

    def _validate_operation(self):
        if set(self._obj.dims) - {"geometry"}:
            raise ValueError(
                "Extra dimensions found. Please slice/squeeze data"
                " so that the only dimension is 'geometry'."
            )

    def to_geodataframe(self):
        """
        Convert to a GeoDataFrame.

        Returns
        -------
        :obj:`geopandas.GeoDataFrame`

        """
        self._validate_operation()
        out_obj = self._obj.drop("crs")
        extra_coords = list(set(list(out_obj.coords)) - {"geometry"})
        if extra_coords:
            out_obj = out_obj.copy().reset_coords(extra_coords)
        geodf = gpd.GeoDataFrame(out_obj.to_dataframe().reset_index())
        geodf.crs = self._obj.coords["crs"].attrs
        return geodf

    def to_netcdf(self, *args, **kwargs):
        """
        Write the data to a netCDF file.
        """
        out_obj = self._obj.copy()
        out_obj.coords["geometry"] = list(map(dumps, out_obj.coords["geometry"].values))
        out_obj.to_netcdf(*args, **kwargs)


@xarray.register_dataarray_accessor("vector")
class VectorArray(BaseVectorX):
    """This is the vector GIS extension for :class:`xarray.DataArray`"""

    def plot(self):
        self.to_geodataframe().plot(column=self._obj.name)


@xarray.register_dataset_accessor("vector")
class VectorDataset(BaseVectorX):
    """This is the vector GIS extension for :class:`xarray.Dataset`"""

    def plot(self, column):
        self._obj[column].vector.plot()



import json
import os

import geopandas as gpd
import pandas
import pytest
import xarray
from numpy.testing import assert_almost_equal
from shapely.geometry import mapping
from shapely.wkt import loads

from geocube.api.core import make_geocube
from geocube.exceptions import VectorDataError
from test.conftest import TEST_COMPARE_DATA_DIR, TEST_INPUT_DATA_DIR

TEST_GARS_PROJ = "+init=epsg:32615"
TEST_GARS_POLY = loads(
    "POLYGON (("
    "-90.58343333333333 41.48343333333334, "
    "-90.59989999999999 41.48343333333334, "
    "-90.59989999999999 41.4999, "
    "-90.58343333333333 41.4999, "
    "-90.58343333333333 41.48343333333334"
    "))"
)


@pytest.mark.parametrize(
    "input_geodata",
    [
        os.path.join(TEST_INPUT_DATA_DIR, "soil_data_flat.geojson"),
        gpd.read_file(os.path.join(TEST_INPUT_DATA_DIR, "soil_data_flat.geojson")),
        pandas.DataFrame(
            gpd.read_file(os.path.join(TEST_INPUT_DATA_DIR, "soil_data_flat.geojson"))
        ),
    ],
)
def test_make_geocube(input_geodata, tmpdir):
    soil_attribute_list = [
        "om_r",
        "sandtotal_r",
        "silttotal_r",
        "claytotal_r",
        "cec7_r",
        "ph1to1h2o_r",
        "dbthirdbar_r",
        "awc_r",
    ]

    out_grid = make_geocube(
        vector_data=input_geodata,
        measurements=soil_attribute_list,
        output_crs=TEST_GARS_PROJ,
        geom=json.dumps(mapping(TEST_GARS_POLY)),
        resolution=(-10, 10),
    )

    # test writing to netCDF
    out_grid.to_netcdf(str(tmpdir.mkdir("make_geocube_soil").join("soil_grid_flat.nc")))

    # test output data
    with xarray.open_dataset(
        os.path.join(TEST_COMPARE_DATA_DIR, "soil_grid_flat.nc"),
        autoclose=True,
        mask_and_scale=False,
    ) as xdc:
        xarray.testing.assert_allclose(out_grid, xdc)


@pytest.mark.parametrize(
    "input_geodata",
    [gpd.read_file(os.path.join(TEST_INPUT_DATA_DIR, "soil_data_flat.geojson"))],
)
def test_make_geocube__categorical(input_geodata, tmpdir):
    input_geodata["soil_type"] = [
        "sand",
        "silt",
        "clay",
        "frank",
        "silt",
        "clay",
        "sand",
    ]
    out_grid = make_geocube(
        vector_data=input_geodata,
        output_crs=TEST_GARS_PROJ,
        geom=json.dumps(mapping(TEST_GARS_POLY)),
        resolution=(-10, 10),
        categorical_enums={"soil_type": ("sand", "silt", "clay")},
    )
    # test writing to netCDF
    out_grid.to_netcdf(
        str(tmpdir.mkdir("make_geocube_soil").join("soil_grid_flat_categorical.nc"))
    )

    # test output data
    with xarray.open_dataset(
        os.path.join(TEST_COMPARE_DATA_DIR, "soil_grid_flat_categorical.nc"),
        autoclose=True,
        mask_and_scale=False,
    ) as xdc:
        xarray.testing.assert_allclose(out_grid, xdc)


@pytest.mark.parametrize(
    "input_geodata",
    [
        os.path.join(TEST_INPUT_DATA_DIR, "soil_data_flat.geojson"),
        gpd.read_file(os.path.join(TEST_INPUT_DATA_DIR, "soil_data_flat.geojson")),
    ],
)
def test_make_geocube__interpolate_na(input_geodata, tmpdir):
    soil_attribute_list = [
        "om_r",
        "sandtotal_r",
        "silttotal_r",
        "claytotal_r",
        "cec7_r",
        "ph1to1h2o_r",
        "dbthirdbar_r",
        "awc_r",
    ]

    out_grid = make_geocube(
        vector_data=input_geodata,
        measurements=soil_attribute_list,
        output_crs=TEST_GARS_PROJ,
        geom=json.dumps(mapping(TEST_GARS_POLY)),
        resolution=(-10, 10),
        interpolate_na_method="nearest",
    )

    # test writing to netCDF
    out_grid.to_netcdf(
        str(tmpdir.mkdir("make_geocube_soil").join("soil_grid_flat_interpolate_na.nc"))
    )

    # test output data
    with xarray.open_dataset(
        os.path.join(TEST_COMPARE_DATA_DIR, "soil_grid_flat_interpolate_na.nc"),
        autoclose=True,
        mask_and_scale=False,
    ) as xdc:
        xarray.testing.assert_allclose(out_grid, xdc)


@pytest.mark.parametrize(
    "input_geodata",
    [
        os.path.join(TEST_INPUT_DATA_DIR, "soil_data_flat.geojson"),
        gpd.read_file(os.path.join(TEST_INPUT_DATA_DIR, "soil_data_flat.geojson")),
    ],
)
def test_make_geocube__like(input_geodata, tmpdir):
    soil_attribute_list = [
        "om_r",
        "sandtotal_r",
        "silttotal_r",
        "claytotal_r",
        "cec7_r",
        "ph1to1h2o_r",
        "dbthirdbar_r",
        "awc_r",
    ]

    with xarray.open_dataset(
        os.path.join(TEST_COMPARE_DATA_DIR, "soil_grid_flat.nc"),
        autoclose=True,
        mask_and_scale=False,
    ) as xdc:
        out_grid = make_geocube(
            vector_data=input_geodata, measurements=soil_attribute_list, like=xdc
        )

        # test writing to netCDF
        out_grid.to_netcdf(
            str(tmpdir.mkdir("make_geocube_soil").join("soil_grid_flat.nc"))
        )
        xarray.testing.assert_allclose(out_grid, xdc)


@pytest.mark.parametrize(
    "input_geodata",
    [
        os.path.join(TEST_INPUT_DATA_DIR, "soil_data_flat.geojson"),
        gpd.read_file(os.path.join(TEST_INPUT_DATA_DIR, "soil_data_flat.geojson")),
    ],
)
def test_make_geocube__only_resolution(input_geodata, tmpdir):
    soil_attribute_list = [
        "om_r",
        "sandtotal_r",
        "silttotal_r",
        "claytotal_r",
        "cec7_r",
        "ph1to1h2o_r",
        "dbthirdbar_r",
        "awc_r",
    ]

    out_grid = make_geocube(
        vector_data=input_geodata,
        measurements=soil_attribute_list,
        resolution=(-0.001, 0.001),
    )

    # test writing to netCDF
    out_grid.to_netcdf(
        str(tmpdir.mkdir("make_geocube_soil").join("soil_grid_flat_original_crs.nc"))
    )

    # test output data
    with xarray.open_dataset(
        os.path.join(TEST_COMPARE_DATA_DIR, "soil_grid_flat_original_crs.nc"),
        autoclose=True,
        mask_and_scale=False,
    ) as xdc:
        xarray.testing.assert_allclose(out_grid, xdc)


@pytest.mark.parametrize(
    "input_geodata",
    [
        os.path.join(TEST_INPUT_DATA_DIR, "time_vector_data.geojson"),
        gpd.read_file(os.path.join(TEST_INPUT_DATA_DIR, "time_vector_data.geojson")),
    ],
)
def test_make_geocube__convert_time(input_geodata, tmpdir):
    out_grid = make_geocube(
        vector_data=input_geodata,
        measurements=["test_attr", "test_time_attr", "test_str_attr"],
        datetime_measurements=["test_time_attr"],
        resolution=(-0.00001, 0.00001),
    )

    # test writing to netCDF
    out_grid.to_netcdf(str(tmpdir.mkdir("geocube_time").join("time_vector_data.nc")))

    # test output data
    with xarray.open_dataset(
        os.path.join(TEST_COMPARE_DATA_DIR, "time_vector_data.nc"),
        autoclose=True,
        mask_and_scale=False,
    ) as xdc:
        xarray.testing.assert_allclose(out_grid, xdc)

    assert out_grid.test_time_attr.attrs["units"] == "seconds from 1970-01-01T00:00:00"
    assert out_grid.test_time_attr.attrs["_FillValue"] == 0


@pytest.mark.parametrize(
    "load_extra_kwargs",
    [
        {"output_crs": "+init=epsg:4326"},
        {"resolution": (-10, 10)},
        {"align": (0, 0)},
        {"output_crs": "+init=epsg:4326", "resolution": (-10, 10), "align": (0, 0)},
    ],
)
def test_make_geocube__like_error_invalid_args(load_extra_kwargs):
    soil_attribute_list = [
        "om_r",
        "sandtotal_r",
        "silttotal_r",
        "claytotal_r",
        "cec7_r",
        "ph1to1h2o_r",
        "dbthirdbar_r",
        "awc_r",
    ]

    with xarray.open_dataset(
        os.path.join(TEST_COMPARE_DATA_DIR, "soil_grid_flat.nc"),
        autoclose=True,
        mask_and_scale=False,
    ) as xdc:
        with pytest.raises(AssertionError):
            make_geocube(
                vector_data=os.path.join(TEST_INPUT_DATA_DIR, "soil_data_flat.geojson"),
                measurements=soil_attribute_list,
                like=xdc,
                **load_extra_kwargs
            )


@pytest.mark.parametrize(
    "input_geodata",
    [
        os.path.join(TEST_INPUT_DATA_DIR, "soil_data_flat.geojson"),
        gpd.read_file(os.path.join(TEST_INPUT_DATA_DIR, "soil_data_flat.geojson")),
    ],
)
def test_make_geocube__no_measurements(input_geodata, tmpdir):
    out_grid = make_geocube(
        vector_data=input_geodata,
        output_crs=TEST_GARS_PROJ,
        geom=json.dumps(mapping(TEST_GARS_POLY)),
        resolution=(-10, 10),
    )

    # test writing to netCDF
    out_grid.to_netcdf(str(tmpdir.mkdir("make_geocube_soil").join("soil_grid_flat.nc")))

    # test output data
    with xarray.open_dataset(
        os.path.join(TEST_COMPARE_DATA_DIR, "soil_grid_flat.nc"),
        autoclose=True,
        mask_and_scale=False,
    ) as xdc:
        xarray.testing.assert_allclose(out_grid, xdc)

    tmpdir.remove()


def test_make_geocube__no_geom(tmpdir):
    out_grid = make_geocube(
        vector_data=os.path.join(TEST_INPUT_DATA_DIR, "soil_data_flat.geojson"),
        measurements=["sandtotal_r"],
        resolution=(-0.001, 0.001),
    )

    # test writing to netCDF
    out_grid.to_netcdf(
        str(tmpdir.mkdir("make_geocube_soil").join("soil_grid_flat_no_geom.nc"))
    )

    # test output data
    with xarray.open_dataset(
        os.path.join(TEST_COMPARE_DATA_DIR, "soil_grid_flat_no_geom.nc"),
        autoclose=True,
        mask_and_scale=False,
    ) as xdc:
        xarray.testing.assert_allclose(out_grid, xdc)

    tmpdir.remove()


@pytest.mark.parametrize(
    "input_geodata",
    [
        gpd.GeoDataFrame(columns=["test_col", "geometry"]),
        gpd.GeoDataFrame(),
        gpd.read_file(os.path.join(TEST_INPUT_DATA_DIR, "soil_data_flat.geojson")).drop(
            columns="geometry"
        ),
    ],
)
def test_make_geocube__invalid_gdf(input_geodata):
    with pytest.raises(VectorDataError):
        make_geocube(vector_data=input_geodata, resolution=(-0.001, 0.001))


def test_make_geocube__no_resolution_error():
    with pytest.raises(RuntimeError):
        make_geocube(
            vector_data=os.path.join(TEST_INPUT_DATA_DIR, "soil_data_flat.geojson"),
            measurements=["sandtotal_r"],
            output_crs=TEST_GARS_PROJ,
            geom=json.dumps(mapping(TEST_GARS_POLY)),
        )


@pytest.mark.parametrize(
    "input_geodata",
    [
        os.path.join(TEST_INPUT_DATA_DIR, "soil_data_group.geojson"),
        gpd.read_file(os.path.join(TEST_INPUT_DATA_DIR, "soil_data_group.geojson")),
    ],
)
def test_make_geocube__group_by(input_geodata, tmpdir):
    soil_attribute_list = [
        "cokey",
        "mukey",
        "drclassdcd",
        "hzdept_r",
        "hzdepb_r",
        "sandtotal_r",
        "silttotal_r",
        "claytotal_r",
    ]
    out_grid = make_geocube(
        vector_data=input_geodata,
        measurements=soil_attribute_list,
        output_crs=TEST_GARS_PROJ,
        geom=json.dumps(mapping(TEST_GARS_POLY)),
        group_by="hzdept_r",
        resolution=(-10, 10),
    )

    # test writing to netCDF
    out_grid.to_netcdf(
        str(tmpdir.mkdir("make_geocube_soil").join("soil_grid_group.nc"))
    )

    # test output data
    with xarray.open_dataset(
        os.path.join(TEST_COMPARE_DATA_DIR, "soil_grid_group.nc"),
        autoclose=True,
        mask_and_scale=False,
    ) as xdc:
        xarray.testing.assert_allclose(out_grid, xdc)

    tmpdir.remove()


@pytest.mark.parametrize(
    "input_geodata",
    [gpd.read_file(os.path.join(TEST_INPUT_DATA_DIR, "soil_data_group.geojson"))],
)
def test_make_geocube__group_by__categorical(input_geodata, tmpdir):
    input_geodata["soil_type"] = [
        "sand",
        "bob",
        "clay",
        "sand",
        "silt",
        "clay",
        "sand",
    ] * 11
    soil_attribute_list = ["sandtotal_r", "silttotal_r", "soil_type", "claytotal_r"]
    out_grid = make_geocube(
        vector_data=input_geodata,
        measurements=soil_attribute_list,
        output_crs=TEST_GARS_PROJ,
        geom=json.dumps(mapping(TEST_GARS_POLY)),
        group_by="hzdept_r",
        resolution=(-10, 10),
        categorical_enums={"soil_type": ("sand", "silt", "clay")},
    )

    # test writing to netCDF
    out_grid.to_netcdf(
        str(tmpdir.mkdir("make_geocube_soil").join("soil_grid_group_categorical.nc"))
    )

    # test output data
    with xarray.open_dataset(
        os.path.join(TEST_COMPARE_DATA_DIR, "soil_grid_group_categorical.nc"),
        autoclose=True,
        mask_and_scale=False,
    ) as xdc:
        xarray.testing.assert_allclose(out_grid, xdc)

    tmpdir.remove()


@pytest.mark.parametrize(
    "input_geodata",
    [
        os.path.join(TEST_INPUT_DATA_DIR, "soil_data_group.geojson"),
        gpd.read_file(os.path.join(TEST_INPUT_DATA_DIR, "soil_data_group.geojson")),
    ],
)
def test_make_geocube__group_by_like(input_geodata, tmpdir):
    soil_attribute_list = [
        "cokey",
        "mukey",
        "drclassdcd",
        "hzdept_r",
        "hzdepb_r",
        "sandtotal_r",
        "silttotal_r",
        "claytotal_r",
    ]

    with xarray.open_dataset(
        os.path.join(TEST_COMPARE_DATA_DIR, "soil_grid_group.nc"),
        autoclose=True,
        mask_and_scale=False,
    ) as xdc:
        out_grid = make_geocube(
            vector_data=input_geodata,
            measurements=soil_attribute_list,
            group_by="hzdept_r",
            like=xdc,
        )

        # test writing to netCDF
        out_grid.to_netcdf(
            str(tmpdir.mkdir("make_geocube_soil").join("soil_grid_group.nc"))
        )

        xarray.testing.assert_allclose(out_grid, xdc)

    tmpdir.remove()


@pytest.mark.parametrize(
    "input_geodata",
    [
        os.path.join(TEST_INPUT_DATA_DIR, "soil_data_group.geojson"),
        gpd.read_file(os.path.join(TEST_INPUT_DATA_DIR, "soil_data_group.geojson")),
    ],
)
def test_make_geocube__group_by_only_resolution(input_geodata, tmpdir):
    soil_attribute_list = ["sandtotal_r", "silttotal_r", "claytotal_r"]

    out_grid = make_geocube(
        vector_data=input_geodata,
        measurements=soil_attribute_list,
        group_by="hzdept_r",
        resolution=(-0.001, 0.001),
    )

    # test writing to netCDF
    out_grid.to_netcdf(
        str(tmpdir.mkdir("make_geocube_soil").join("soil_grid_grouped_original_crs.nc"))
    )

    # test output data
    with xarray.open_dataset(
        os.path.join(TEST_COMPARE_DATA_DIR, "soil_grid_grouped_original_crs.nc"),
        autoclose=True,
        mask_and_scale=False,
    ) as xdc:
        xarray.testing.assert_allclose(out_grid, xdc)

    tmpdir.remove()


@pytest.mark.parametrize(
    "input_geodata",
    [
        os.path.join(TEST_INPUT_DATA_DIR, "time_vector_data.geojson"),
        gpd.read_file(os.path.join(TEST_INPUT_DATA_DIR, "time_vector_data.geojson")),
    ],
)
def test_make_geocube__group_by_time(input_geodata, tmpdir):
    out_grid = make_geocube(
        vector_data=input_geodata,
        datetime_measurements=["test_time_attr"],
        resolution=(-0.00001, 0.00001),
        group_by="test_time_attr",
    )

    # test writing to netCDF
    out_grid.to_netcdf(
        str(tmpdir.mkdir("make_geocube_time").join("vector_time_data_group.nc"))
    )
    # test output data
    with xarray.open_dataset(
        os.path.join(TEST_COMPARE_DATA_DIR, "vector_time_data_group.nc"),
        autoclose=True,
        mask_and_scale=False,
    ) as xdc:
        xarray.testing.assert_allclose(out_grid, xdc)

    tmpdir.remove()


@pytest.mark.parametrize(
    "input_geodata",
    [
        os.path.join(TEST_INPUT_DATA_DIR, "time_vector_data.geojson"),
        gpd.read_file(os.path.join(TEST_INPUT_DATA_DIR, "time_vector_data.geojson")),
    ],
)
def test_make_geocube__group_by_convert_with_time(input_geodata, tmpdir):
    out_grid = make_geocube(
        vector_data=input_geodata,
        datetime_measurements=["test_time_attr"],
        resolution=(-0.00001, 0.00001),
        group_by="test_attr",
    )

    # test writing to netCDF
    out_grid.to_netcdf(
        str(tmpdir.mkdir("make_geocube_time").join("vector_data_group.nc"))
    )

    # test output data
    with xarray.open_dataset(
        os.path.join(TEST_COMPARE_DATA_DIR, "vector_data_group.nc"),
        autoclose=True,
        mask_and_scale=False,
    ) as xdc:
        xarray.testing.assert_allclose(out_grid, xdc)

    assert out_grid.test_time_attr.attrs["units"] == "seconds from 1970-01-01T00:00:00"
    assert out_grid.test_time_attr.attrs["_FillValue"] == 0

    tmpdir.remove()


@pytest.mark.parametrize(
    "load_extra_kwargs",
    [
        {"output_crs": "+init=epsg:4326"},
        {"resolution": (-10, 10)},
        {"align": (0, 0)},
        {"output_crs": "+init=epsg:4326", "resolution": (-10, 10), "align": (0, 0)},
    ],
)
def test_make_geocube__group_by_like_error_invalid_args(load_extra_kwargs):
    soil_attribute_list = [
        "cokey",
        "mukey",
        "drclassdcd",
        "hzdept_r",
        "hzdepb_r",
        "sandtotal_r",
        "silttotal_r",
        "claytotal_r",
    ]

    with xarray.open_dataset(
        os.path.join(TEST_COMPARE_DATA_DIR, "soil_grid_group.nc"),
        autoclose=True,
        mask_and_scale=False,
    ) as xdc:
        with pytest.raises(AssertionError):
            make_geocube(
                vector_data=os.path.join(
                    TEST_INPUT_DATA_DIR, "soil_data_group.geojson"
                ),
                measurements=soil_attribute_list,
                like=xdc,
                group_by="hzdept_r",
                **load_extra_kwargs
            )


@pytest.mark.parametrize(
    "input_geodata",
    [
        os.path.join(TEST_INPUT_DATA_DIR, "soil_data_group.geojson"),
        gpd.read_file(os.path.join(TEST_INPUT_DATA_DIR, "soil_data_group.geojson")),
    ],
)
def test_make_geocube__group_by_no_measurements(input_geodata, tmpdir):
    out_grid = make_geocube(
        vector_data=input_geodata,
        output_crs=TEST_GARS_PROJ,
        geom=json.dumps(mapping(TEST_GARS_POLY)),
        group_by="hzdept_r",
        resolution=(-10, 10),
    )

    # test writing to netCDF
    out_grid.to_netcdf(
        str(tmpdir.mkdir("make_geocube_soil").join("soil_grid_group.nc"))
    )

    # test output data
    with xarray.open_dataset(
        os.path.join(TEST_COMPARE_DATA_DIR, "soil_grid_group.nc"),
        autoclose=True,
        mask_and_scale=False,
    ) as xdc:
        xarray.testing.assert_allclose(out_grid, xdc)

    tmpdir.remove()


def test_make_geocube__group_by__no_geom(tmpdir):
    out_grid = make_geocube(
        vector_data=os.path.join(TEST_INPUT_DATA_DIR, "soil_data_group.geojson"),
        measurements=["sandtotal_r"],
        group_by="hzdept_r",
        resolution=(-0.001, 0.001),
    )

    # test writing to netCDF
    out_grid.to_netcdf(
        str(tmpdir.mkdir("make_geocube_soil").join("soil_grid_group_no_geom.nc"))
    )

    # test output data
    with xarray.open_dataset(
        os.path.join(TEST_COMPARE_DATA_DIR, "soil_grid_group_no_geom.nc"),
        autoclose=True,
        mask_and_scale=False,
    ) as xdc:
        xarray.testing.assert_allclose(out_grid, xdc)

    tmpdir.remove()


def test_make_geocube__group_by__no_resolution_error():
    with pytest.raises(RuntimeError):
        make_geocube(
            vector_data=os.path.join(TEST_INPUT_DATA_DIR, "soil_data_group.geojson"),
            measurements=["sandtotal_r"],
            output_crs=TEST_GARS_PROJ,
            geom=json.dumps(mapping(TEST_GARS_POLY)),
            group_by="hzdept_r",
        )


def test_make_geocube__new_bounds_crs():
    utm_cube = make_geocube(
        vector_data=os.path.join(TEST_INPUT_DATA_DIR, "wgs84_geom.geojson"),
        output_crs="epsg:32614",
        resolution=(-1, 1),
    )
    assert_almost_equal(
        utm_cube.id.rio.bounds(), (1665478.0, 7018306.0, 1665945.0, 7018509.0)
    )



import click
from mock import MagicMock, patch

from geocube.cli.geocube import check_version


@patch("geocube.cli.geocube.__version__", "1.1")
def test_check_version(capsys):
    context = MagicMock(click.Context)
    context.resilient_parsing = False
    check_version(context, "str", "str")

    stdout = capsys.readouterr()[0]
    assert "geocube v1.1" in stdout



import os

import pytest
import xarray
from click.testing import CliRunner
from mock import patch

from geocube.cli.geocube import geocube
from test.conftest import TEST_COMPARE_DATA_DIR, TEST_INPUT_DATA_DIR


def _get_called_dict(**kwargs):
    default = dict(
        align=None,
        fill=None,
        geom=None,
        group_by=None,
        interpolate_na_method=None,
        like=None,
        measurements=None,
        output_crs=None,
        resolution=None,
        vector_data=os.path.join(TEST_INPUT_DATA_DIR, "soil_data_flat.geojson"),
    )
    default.update(**kwargs)
    return default


INPUT_GEOM = (
    '{"type": "Polygon", "coordinates": '
    "[[[50.5, 50.0], [50.5, 50.5], [50.0, 50.5], [50.0, 50.0], [50.5, 50.0]]]}"
)
LIKE_PATH = os.path.join(TEST_COMPARE_DATA_DIR, "soil_grid_flat.nc")


@pytest.mark.parametrize(
    "params, called_with",
    [
        (
            ["-m", "measure1", "--measurements", "measure2"],
            _get_called_dict(measurements=("measure1", "measure2")),
        ),
        (["-c", "+init=epsg:4326"], _get_called_dict(output_crs="+init=epsg:4326")),
        (
            ["--output-crs", "+init=epsg:4326"],
            _get_called_dict(output_crs="+init=epsg:4326"),
        ),
        (["-r", "-10", "10"], _get_called_dict(resolution=(-10, 10))),
        (["--resolution", "-10", "10"], _get_called_dict(resolution=(-10, 10))),
        (["-a", "-10", "10"], _get_called_dict(align=(-10, 10))),
        (["--align", "-10", "10"], _get_called_dict(align=(-10, 10))),
        (["-g", INPUT_GEOM], _get_called_dict(geom=INPUT_GEOM)),
        (["--geom", INPUT_GEOM], _get_called_dict(geom=INPUT_GEOM)),
        (
            ["-l", LIKE_PATH],
            _get_called_dict(like=xarray.open_dataset(LIKE_PATH, autoclose=True)),
        ),
        (
            ["--like", LIKE_PATH],
            _get_called_dict(like=xarray.open_dataset(LIKE_PATH, autoclose=True)),
        ),
        (["-f", "-10"], _get_called_dict(fill=-10)),
        (["--fill", "-10"], _get_called_dict(fill=-10)),
        (["-gb", "attr"], _get_called_dict(group_by="attr")),
        (["--group-by", "attr"], _get_called_dict(group_by="attr")),
        (["-i", "nearest"], _get_called_dict(interpolate_na_method="nearest")),
        (
            ["--interpolate-na-method", "nearest"],
            _get_called_dict(interpolate_na_method="nearest"),
        ),
    ],
)
@patch("geocube.cli.commands.make_geocube.core.make_geocube")
def test_make_geocube_params(make_geocube_mock, params, called_with):
    with patch("geocube.cli.commands.make_geocube.click.Path"):
        cmd_out = CliRunner().invoke(
            geocube,
            [
                "make-geocube",
                os.path.join(TEST_INPUT_DATA_DIR, "soil_data_flat.geojson"),
                "outfile.nc",
            ]
            + params,
        )
    try:
        make_geocube_mock.assert_called_with(**called_with)
        make_geocube_mock().to_netcdf.assert_called_with("outfile.nc")
    except AssertionError:
        print(cmd_out.output)
        raise



import os

TEST_DATA_DIR = os.path.join(os.path.dirname(__file__), "test_data")
TEST_INPUT_DATA_DIR = os.path.join(TEST_DATA_DIR, "input")
TEST_COMPARE_DATA_DIR = os.path.join(TEST_DATA_DIR, "compare")
